package dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import entites.Activite;
import entites.Adresse;
import entites.Personne;

public class Dao implements IDao {

	@PersistenceContext
	private EntityManager em;

	// personnes

	public Personne getPersonne(Long categorieId) {
		return em.find(Personne.class, categorieId);
	}

	@SuppressWarnings("unchecked")
	public List<Personne> getAllPersonnes() {
		return em.createQuery("select p from Personne p").getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<Personne> getAllPersonnesWithNomLike(String modeleNom) {
		return em.createQuery("select p from Personne p where p.nom like :modele").setParameter("modele", modeleNom).getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<Activite> getActivitesOfPersonne(Long personneId) {
		return em.createQuery("select a from Personne p join p.activites a where p.id=:personneId").setParameter("personneId", personneId)
				.getResultList();
	}

	public Personne updatePersonne(Personne personne) {
		return em.merge(personne);
	}

	public Personne savePersonne(Personne personne) {
		em.persist(personne);
		return personne;
	}

	public void deletePersonne(Long personneId) {
		Personne personne = em.find(Personne.class, personneId);
		if (personne == null) {
			throw new DaoException(30);
		}
		em.remove(personne);
	}

	// activits

	public Activite getActivite(Long activiteId) {
		return em.find(Activite.class, activiteId);
	}

	@SuppressWarnings("unchecked")
	public List<Activite> getAllActivites() {
		return em.createQuery("select a from Activite a").getResultList();
	}

	@SuppressWarnings("unchecked")
	public List<Activite> getAllActivitesWithNomLike(String modeleNom) {
		return em.createQuery("select a from Activite a where a.nom like :modele").setParameter("modele", modeleNom).getResultList();
	}

	public Activite saveActivite(Activite activite) {
		em.persist(activite);
		return activite;
	}

	public Activite updateActivite(Activite activite) {
		return em.merge(activite);
	}

	public void deleteActivite(Long activiteId) {
		Activite article = em.find(Activite.class, activiteId);
		if (article == null) {
			throw new DaoException(20);
		}
		em.remove(article);
	}

	@SuppressWarnings("unchecked")
	public List<Personne> getPersonnesDoingActivite(Long activiteId) {
		// la requte suivante marche avec Hibernate, pas avec Toplink qui refuse la navigation p.activites.id
		// return em.createQuery("select p from Personne p, Activite a where p.activites.id=a.id and a.id=:activiteId").setParameter("activiteId",
		// activiteId).getResultList();

		// la requte suivante est accepte par les deux
		return em.createQuery("select p from Personne p join p.activites a where a.id=:activiteId").setParameter("activiteId", activiteId)
				.getResultList();
	}

	// adresses
	@SuppressWarnings("unchecked")
	public List<Adresse> getAllAdresses() {
		return em.createQuery("select a from Adresse a").getResultList();
	}

}
