package web;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import service.IService;
import dao.DaoException;
import entites.Personne;

@SuppressWarnings("serial")
public class Application extends HttpServlet {
	// paramtres d'instance
	private String urlErreurs = null;

	private ArrayList erreursInitialisation = new ArrayList<String>();

	private String[] paramtres = { "urlList", "urlEdit", "urlErreurs" };

	private Map params = new HashMap<String, String>();

	// service
	private IService service = null;

	// init
	@SuppressWarnings("unchecked")
	public void init() throws ServletException {
		// on rcupre les paramtres d'initialisation de la servlet
		ServletConfig config = getServletConfig();
		// on traite les autres paramtres d'initialisation
		String valeur = null;
		for (int i = 0; i < paramtres.length; i++) {
			// valeur du paramtre
			valeur = config.getInitParameter(paramtres[i]);
			// paramtre prsent ?
			if (valeur == null) {
				// on note l'erreur
				erreursInitialisation.add("Le paramtre [" + paramtres[i]
						+ "] n'a pas t initialis");
			} else {
				// on mmorise la valeur du paramtre
				params.put(paramtres[i], valeur);
			}
		}
		// l'url de la vue [erreurs] a un traitement particulier
		urlErreurs = config.getInitParameter("urlErreurs");
		if (urlErreurs == null)
			throw new ServletException(
					"Le paramtre [urlErreurs] n'a pas t initialis");
		// configuration de l'application
		ApplicationContext ctx = new ClassPathXmlApplicationContext(
				"spring-config.xml");
		// couche service
		service = (IService) ctx.getBean("service");
		// on vide la base
		clean();
		// on la remplit
		try {
			fill();
		} catch (ParseException e) {
			throw new ServletException(e);
		}
	}

	// remplissage table
	public void fill() throws ParseException {
		// cration personnes
		Personne p1 = new Personne("p1", "Paul", new SimpleDateFormat("dd/MM/yy")
				.parse("31/01/2000"), true, 2);
		Personne p2 = new Personne("p2", "Sylvie", new SimpleDateFormat("dd/MM/yy")
				.parse("05/07/2001"), false, 0);
		// qu'on sauvegarde
		service.saveArray(new Personne[] { p1, p2 });
	}

	// supression lments de la table
	public void clean() {
		for (Personne p : service.getAll()) {
			service.deleteOne(p.getId());
		}
	}

	// GET
	@SuppressWarnings("unchecked")
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {

		// on vrifie comment s'est passe l'initialisation de la servlet
		if (erreursInitialisation.size() != 0) {
			// on passe la main  la page d'erreurs
			request.setAttribute("erreurs", erreursInitialisation);
			getServletContext().getRequestDispatcher(urlErreurs).forward(request,
					response);
			// fin
			return;
		}
		// on rcupre la mthode d'envoi de la requte
		String mthode = request.getMethod().toLowerCase();
		// on rcupre l'action  excuter
		String action = request.getPathInfo();
		// action ?
		if (action == null) {
			action = "/list";
		}
		// excution action
		if (mthode.equals("get") && action.equals("/list")) {
			// liste des personnes
			doListPersonnes(request, response);
			return;
		}
		if (mthode.equals("get") && action.equals("/delete")) {
			// suppression d'une personne
			doDeletePersonne(request, response);
			return;
		}
		if (mthode.equals("get") && action.equals("/edit")) {
			// prsentation formulaire ajout / modification d'une personne
			doEditPersonne(request, response);
			return;
		}
		if (mthode.equals("post") && action.equals("/validate")) {
			// validation formulaire ajout / modification d'une personne
			doValidatePersonne(request, response);
			return;
		}
		// autres cas
		doListPersonnes(request, response);
	}

	// affichage liste des personnes
	private void doListPersonnes(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// le modle de la vue [list]
		request.setAttribute("personnes", service.getAll());
		// affichage de la vue [list]
		getServletContext().getRequestDispatcher((String) params.get("urlList"))
				.forward(request, response);
	}

	// modification / ajout d'une personne
	private void doEditPersonne(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// on rcupre l'id de la personne
		int id = Integer.parseInt(request.getParameter("id"));
		// ajout ou modification ?
		Personne personne = null;
		if (id != -1) {
			// modification - on rcupre la personne  modifier
			personne = service.getOne(id);
			if (personne == null) {
				throw new DaoException(String.format(
						"La personne n [%d] n'existe pas", id), 2);
			}
			request.setAttribute("id", personne.getId());
			request.setAttribute("version", personne.getVersion());
		} else {
			// ajout - on cre une personne vide
			personne = new Personne();
			request.setAttribute("id", -1);
			request.setAttribute("version", -1);
		}
		// on la met l'objet [Personne] dans la session de l'utilisateur
		request.getSession().setAttribute("personne", personne);
		// et dans le modle de la vue [edit]
		request.setAttribute("erreurEdit", "");
		request.setAttribute("prenom", personne.getPrenom());
		request.setAttribute("nom", personne.getNom());
		Date dateNaissance = personne.getDatenaissance();
		if (dateNaissance != null) {
			request.setAttribute("datenaissance", new SimpleDateFormat("dd/MM/yyyy")
					.format(dateNaissance));
		} else {
			request.setAttribute("datenaissance", "");
		}
		request.setAttribute("marie", personne.isMarie());
		request.setAttribute("nbenfants", personne.getNbenfants());
		// affichage de la vue [edit]
		getServletContext().getRequestDispatcher((String) params.get("urlEdit"))
				.forward(request, response);
	}

	// suppression d'une personne
	private void doDeletePersonne(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// on rcupre l'id de la personne
		int id = Integer.parseInt(request.getParameter("id"));
		// on supprime la personne
		service.deleteOne(id);
		// on redirige vers la liste des personnes
		response.sendRedirect("list");
	}

	// validation modification / ajout d'une personne
	public void doValidatePersonne(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// on rcupre les lments posts
		boolean formulaireErron = false;
		boolean erreur;
		// le prnom
		String prenom = request.getParameter("prenom").trim();
		// prnom valide ?
		if (prenom.length() == 0) {
			// on note l'erreur
			request.setAttribute("erreurPrenom", "Le prnom est obligatoire");
			formulaireErron = true;
		}
		// le nom
		String nom = request.getParameter("nom").trim();
		// prnom valide ?
		if (nom.length() == 0) {
			// on note l'erreur
			request.setAttribute("erreurNom", "Le nom est obligatoire");
			formulaireErron = true;
		}
		// la date de naissance
		Date datenaissance = null;
		try {
			datenaissance = new SimpleDateFormat("dd/MM/yyyy").parse(request
					.getParameter("datenaissance").trim());
		} catch (ParseException e) {
			// on note l'erreur
			request.setAttribute("erreurDateNaissance", "Date incorrecte");
			formulaireErron = true;
		}
		// tat marital
		boolean marie = Boolean.parseBoolean(request.getParameter("marie").trim());
		// nombre d'enfants
		int nbenfants = 0;
		erreur = false;
		try {
			nbenfants = Integer.parseInt(request.getParameter("nbenfants").trim());
			if (nbenfants < 0) {
				erreur = true;
			}
		} catch (NumberFormatException ex) {
			// on note l'erreur
			erreur = true;
		}
		// nombre d'enfants erron ?
		if (erreur) {
			// on signale l'erreur
			request.setAttribute("erreurNbEnfants", "Nombre d'enfants incorrect");
			formulaireErron = true;
		}
		// id de la personne
		int id = Integer.parseInt(request.getParameter("id"));
		// le formulaire est-il erron ?
		if (formulaireErron) {
			// on raffiche le formulaire avec les messages d'erreurs
			showFormulaire(request, response, "");
			// fini
			return;
		}
		// le formulaire est correct - on met  jour la personne qui a t place
		// dans la session
		// avec les informations envoyes par le client
		Personne personne = (Personne) request.getSession()
				.getAttribute("personne");
		personne.setDatenaissance(datenaissance);
		personne.setMarie(marie);
		personne.setNbenfants(nbenfants);
		personne.setNom(nom);
		personne.setPrenom(prenom);
		// persistance
		try {
			if (id == -1) {
				// cration
				service.saveOne(personne);
			} else {
				// mise  jour
				service.updateOne(personne);
			}
		} catch (DaoException ex) {
			// on raffiche le formulaire avec le message de l'erreur survenue
			showFormulaire(request, response, ex.getMessage());
			// fini
			return;
		}
		// on redirige vers la liste des personnes
		response.sendRedirect("list");
	}

	// affichage formulaire pr-rempli
	private void showFormulaire(HttpServletRequest request,
			HttpServletResponse response, String erreurEdit) throws ServletException,
			IOException {
		// on prpare le modle de la vue [edit]
		request.setAttribute("erreurEdit", erreurEdit);
		request.setAttribute("id", request.getParameter("id"));
		request.setAttribute("version", request.getParameter("version"));
		request.setAttribute("prenom", request.getParameter("prenom").trim());
		request.setAttribute("nom", request.getParameter("nom").trim());
		request.setAttribute("datenaissance", request.getParameter("datenaissance")
				.trim());
		request.setAttribute("marie", request.getParameter("marie"));
		request.setAttribute("nbenfants", request.getParameter("nbenfants").trim());
		// affichage de la vue [edit]
		getServletContext().getRequestDispatcher((String) params.get("urlEdit"))
				.forward(request, response);
	}

	// post
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		// on passe la main au GET
		doGet(request, response);
	}

}
