package dao;

import java.util.List;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import entites.Personne;

@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class Dao implements IDao {

	@PersistenceContext
	private EntityManager em;

	// supprimer une personne via son identifiant
	public void deleteOne(Integer id) {
		Personne personne = em.find(Personne.class, id);
		if (personne == null) {
			throw new DaoException(String.format("Personne n[%d] inconnue", id), 2);
		}
		em.remove(personne);
	}

	@SuppressWarnings("unchecked")
	// obtenir toutes les personnes
	public List<Personne> getAll() {
		return em.createQuery("select p from Personne p").getResultList();
	}

	@SuppressWarnings("unchecked")
	// obtenir les personnes dont le nom correspond un modle
	public List<Personne> getAllLike(String modele) {
		return em.createQuery("select p from Personne p where p.nom like :modele")
				.setParameter("modele", modele).getResultList();
	}

	// obtenir une personne via son identifiant
	public Personne getOne(Integer id) {
		return em.find(Personne.class, id);
	}

	// sauvegarder une personne
	public Personne saveOne(Personne personne) {
		em.persist(personne);
		return personne;
	}

	// mettre  jour une personne
	public Personne updateOne(Personne personne) {
		Personne p = em.find(Personne.class, personne.getId());
		if (p == null) {
			throw new DaoException(String.format("Personne n[%d] inconnue", personne.getId()), 2);
		}
		return em.merge(personne);
	}

}
